package nz.ac.auckland.se281;

import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;
import nz.ac.auckland.se281.task1.Greetings;

/**
 *
 * You cannot modify this class!
 *
 */
public class Main {

    public enum PolicyType {
        HOME, CAR, LIFE
    }

    public enum Command {

        // Task 1 commands
        SAY_HELLO(0, "Print a greeting"), //

        // General menu commands
        HELP(0, "Print usage"), //
        EXIT(0, "Exit the application");


        private int numArgs;
        private String message;
        private String[] optionPrompts;

        private Command(int numArgs, String message) {
            this(numArgs, message, new String[] {});
        }

        private Command(int numArgs, String message, String[] optionPrompts) {
            this.numArgs = numArgs;
            this.message = message;
            this.optionPrompts = optionPrompts;
        }

        public boolean hasArguments() {
            return numArgs > 0;
        }

        public int getNumArgs() {
            return numArgs;
        }

        public boolean hasOptions() {
            return optionPrompts.length > 0;
        }

        public int getNumOptions() {
            return optionPrompts.length;
        }

        public String getOptionPrompt(int index) {
            return optionPrompts[index];
        }

        public static String getHelp() {
            StringBuilder sb = new StringBuilder();
            for (Command c : Command.values()) {

                if (c.toString().startsWith("___")) {
                    sb.append(System.lineSeparator());
                    sb.append(c.message + ":");
                    sb.append(System.lineSeparator());
                    sb.append("-----------------------");
                } else {
                    sb.append(c);
                    sb.append("\t");
                    if (c.toString().length() < 8) {
                        sb.append("\t");
                    }
                    sb.append(c.numArgs > 0 ? "[" + c.numArgs + " arguments]" : "[no args]");
                    sb.append("\t");
                    sb.append(c.message);
                }

                sb.append(System.lineSeparator());
            }
            return sb.toString();
        }

    }

        private final Greetings greetings;

        public static Scanner scanner = new Scanner(System.in);
        private static final String COMMAND_PREFIX = "hi-bye> ";

    protected Main(Greetings greetings) {
        this.greetings = greetings;
    }

        public static void main(String[] args) {
            Main cli = new Main( new Greetings());
            cli.start();
        }

        protected void start() {
            printBanner();
            System.out.println(Command.getHelp());
            String command;
            do {
                System.out.print(COMMAND_PREFIX);
                command = scanner.nextLine().trim();
            } while (processCommand(command));
        }

        public static void printBanner() {
            // https://patorjk.com/software/taag/
            // https://www.freeformatter.com/java-dotnet-escape.html#before-output

            StringBuilder buf = new StringBuilder();
            buf.append("   _____  ____  ______ _______ ______ _   _  _____   ___   ___  __ "
                    + "\r\n  / ____|/ __ \\|  ____|__   __|  ____| \\ | |/ ____| |__ \\ / _ \\/_ |"
                    + "\r\n | (___ | |  | | |__     | |  | |__  |  \\| | |  __     ) | (_) || |"
                    + "\r\n  \\___ \\| |  | |  __|    | |  |  __| | . ` | | |_ |   / / > _ < | |"
                    + "\r\n  ____) | |__| | |       | |  | |____| |\\  | |__| |  / /_| (_) || |"
                    + "\r\n |_____/ \\____/|_|       |_|  |______|_| \\_|\\_____| |____|\\___/ |_|"
                    + "\r\n");
            System.out.println(buf.toString());
        }

        private boolean isValidCommand(String command) {
            try {
                Command.valueOf(command);
            } catch (Exception e) {
                return false;
            }
            return true;
        }

        /**
         *
         * @param command
         * @return true if I want another command, false if need to stop the program
         */
        private boolean processCommand(String input) {

            // System.out.println("Processing command: " + input);

            input = input.trim();
            String command = input.contains(" ") ? input.split(" ")[0] : input;

            // add some leniency
            command = command.toUpperCase();
            command = command.replaceAll("-", "_");

            if (!isValidCommand(command)) {
                System.err.println(MessageCli.COMMAND_NOT_FOUND + ": \"" + command + "\"");
                return true;
            }

            Command c = Command.valueOf(command);

            if (!checkArgs(c, input)) {
                int numArgs = c.getNumArgs();
                System.err.println(MessageCli.WRONG_ARGUMENT_COUNT + ". Expected " + c.getNumArgs()
                        + " argument" + (numArgs > 1 ? "s" : "") + " for the \"" + command
                        + "\" command");
                return true;
            }

            String[] args = input.split(" ");
            switch (c) {
                // Task 1
                case SAY_HELLO:
                    greetings.sayHello();
                    break;

                // General menu commands
                case EXIT:
                    MessageCli.END.printMessage();
                    return false;
                case HELP:
                    System.out.println(Command.getHelp());
                    break;
                default:
                    break;
            }
            return true;
        }

        public static String[] options(String... values) {
            List<String> all = new ArrayList<String>();
            all.addAll(List.of(values));
            return all.toArray(new String[all.size()]);
        }

        private boolean checkArgs(Command c, String input) {
            return input.split(" ").length - 1 == c.getNumArgs();
        }
}
